@@ -4,9 +4,13 @@ import math
 import os
 import re
 from datetime import datetime, timedelta
+from email.utils import format_datetime, parsedate_to_datetime
 from typing import Optional
 
+from bravado.exception import HTTPForbidden
+
 from django.contrib.auth.models import Group, User
+from django.core.exceptions import ObjectDoesNotExist
 from django.core.serializers.json import DjangoJSONEncoder
 from django.db import models
 from django.utils.timezone import now
@@ -18,7 +22,7 @@ from allianceauth.authentication.models import CharacterOwnership
 from allianceauth.eveonline.models import EveCorporationInfo
 from allianceauth.notifications import notify
 from allianceauth.services.hooks import get_extension_logger
-from app_utils.allianceauth import notify_admins_throttled, notify_throttled
+from app_utils.allianceauth import notify_admins
 from app_utils.datetime import DATETIME_FORMAT
 from app_utils.helpers import chunks
 from app_utils.logging import LoggerAddTag
@@ -26,6 +30,7 @@ from app_utils.logging import LoggerAddTag
 from .. import __title__, constants
 from ..app_settings import (
     STRUCTURES_ADD_TIMERS,
+    STRUCTURES_ADMIN_NOTIFICATIONS_ENABLED,
     STRUCTURES_DEFAULT_LANGUAGE,
     STRUCTURES_DEVELOPER_MODE,
     STRUCTURES_FEATURE_CUSTOMS_OFFICES,
@@ -33,13 +38,12 @@ from ..app_settings import (
     STRUCTURES_HOURS_UNTIL_STALE_NOTIFICATION,
     STRUCTURES_NOTIFICATION_SYNC_GRACE_MINUTES,
     STRUCTURES_NOTIFICATIONS_ARCHIVING_ENABLED,
-    STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
     STRUCTURES_STRUCTURE_SYNC_GRACE_MINUTES,
 )
-from ..helpers.esi_fetch import esi_fetch, esi_fetch_with_localization
+from ..helpers.esi_fetch import _esi_client, esi_fetch, esi_fetch_with_localization
 from ..managers import OwnerAssetManager, OwnerManager
 from .eveuniverse import EveMoon, EvePlanet, EveSolarSystem, EveType, EveUniverse
-from .notifications import EveEntity, Notification, NotificationType
+from .notifications import EveEntity, Notification, NotificationType, Webhook
 from .structures import PocoDetails, Structure
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
@@ -68,7 +72,29 @@ class General(models.Model):
 class Owner(models.Model):
     """A corporation that owns structures"""
 
-    ESI_CHARACTER_NOTIFICATION_CACHE_DURATION = 600
+    class RotateCharactersType(models.TextChoices):
+        """Type of sync to rotate characters for."""
+
+        STRUCTURES = "structures"
+        NOTIFICATIONS = "notifications"
+
+        @property
+        def last_used_at_name(self) -> str:
+            """Name of last used at property."""
+            if self is self.STRUCTURES:
+                return "structures_last_used_at"
+            elif self is self.NOTIFICATIONS:
+                return "notifications_last_used_at"
+            raise NotImplementedError(f"Not defined for: {self}")
+
+        @property
+        def esi_cache_duration(self) -> int:
+            """ESI cache duration in seconds."""
+            if self is self.STRUCTURES:
+                return 3600
+            elif self is self.NOTIFICATIONS:
+                return 600
+            raise NotImplementedError(f"Not defined for: {self}")
 
     # PK
     corporation = models.OneToOneField(
@@ -84,10 +110,10 @@ class Owner(models.Model):
         help_text=("whether pocos of this owner are shown on public POCO page"),
     )
     assets_last_update_at = models.DateTimeField(
-        null=True, default=None, blank=True, help_text="when the last update happened"
-    )
-    assets_last_update_ok = models.BooleanField(
-        null=True, default=None, help_text="True if the last update was successful"
+        null=True,
+        default=None,
+        blank=True,
+        help_text="when the last successful update happened",
     )
     character_ownership = models.ForeignKey(
         CharacterOwnership,
@@ -99,10 +125,10 @@ class Owner(models.Model):
         help_text="OUTDATED. Has been replaced by OwnerCharacter",
     )
     forwarding_last_update_at = models.DateTimeField(
-        null=True, default=None, blank=True, help_text="when the last sync happened"
-    )
-    forwarding_last_update_ok = models.BooleanField(
-        null=True, default=None, help_text="True if the last update was successful"
+        null=True,
+        default=None,
+        blank=True,
+        help_text="when the last successful update happened",
     )
     has_default_pings_enabled = models.BooleanField(
         default=True,
@@ -129,11 +155,17 @@ class Owner(models.Model):
             "the overall status of this services"
         ),
     )
-    notifications_last_update_at = models.DateTimeField(
-        null=True, default=None, blank=True, help_text="when the last sync happened"
+    is_up = models.BooleanField(
+        null=True,
+        default=None,
+        editable=False,
+        help_text="whether all services for this owner are currently up",
     )
-    notifications_last_update_ok = models.BooleanField(
-        null=True, default=None, help_text="True if the last update was successful"
+    notifications_last_update_at = models.DateTimeField(
+        null=True,
+        default=None,
+        blank=True,
+        help_text="when the last successful update happened",
     )
     ping_groups = models.ManyToManyField(
         Group,
@@ -142,10 +174,10 @@ class Owner(models.Model):
         help_text="Groups to be pinged for each notification - ",
     )
     structures_last_update_at = models.DateTimeField(
-        null=True, default=None, blank=True, help_text="when the last sync happened"
-    )
-    structures_last_update_ok = models.BooleanField(
-        null=True, default=None, help_text="True if the last update was successful"
+        null=True,
+        default=None,
+        blank=True,
+        help_text="when the last successful update happened",
     )
     webhooks = models.ManyToManyField(
         "Webhook",
@@ -166,14 +198,13 @@ class Owner(models.Model):
 
     def save(self, *args, **kwargs) -> None:
         if self.is_alliance_main:
-            Owner.objects.update(is_alliance_main=False)
+            Owner.objects.filter(
+                corporation__alliance_id=self.corporation.alliance_id
+            ).update(is_alliance_main=False)
+            if "update_fields" in kwargs:
+                kwargs["update_fields"].append("is_alliance_main")
         super().save(*args, **kwargs)
 
-    @property
-    def is_structure_sync_ok(self) -> bool:
-        """True if last sync was ok and happend recently, else False."""
-        return self.structures_last_update_ok is True and self.is_structure_sync_fresh
-
     @property
     def is_structure_sync_fresh(self) -> bool:
         """True if last sync happened with grace time, else False."""
@@ -181,14 +212,6 @@ class Owner(models.Model):
             now() - timedelta(minutes=STRUCTURES_STRUCTURE_SYNC_GRACE_MINUTES)
         )
 
-    @property
-    def is_notification_sync_ok(self) -> bool:
-        """True if last sync was ok and happend recently, else False."""
-        return (
-            self.notifications_last_update_ok is True
-            and self.is_notification_sync_fresh
-        )
-
     @property
     def is_notification_sync_fresh(self) -> bool:
         """True if last sync happened with grace time, else False."""
@@ -198,11 +221,6 @@ class Owner(models.Model):
             > (now() - timedelta(minutes=STRUCTURES_NOTIFICATION_SYNC_GRACE_MINUTES))
         )
 
-    @property
-    def is_forwarding_sync_ok(self) -> bool:
-        """True if last sync was ok and happend recently, else False."""
-        return self.forwarding_last_update_ok is True and self.is_forwarding_sync_fresh
-
     @property
     def is_forwarding_sync_fresh(self) -> bool:
         """True if last sync happened with grace time, else False."""
@@ -210,11 +228,6 @@ class Owner(models.Model):
             now() - timedelta(minutes=STRUCTURES_NOTIFICATION_SYNC_GRACE_MINUTES)
         )
 
-    @property
-    def is_assets_sync_ok(self) -> bool:
-        """True if last sync was ok and happend recently, else False."""
-        return self.assets_last_update_ok is True and self.is_assets_sync_fresh
-
     @property
     def is_assets_sync_fresh(self) -> bool:
         """True if last sync happened with grace time, else False."""
@@ -228,12 +241,49 @@ class Owner(models.Model):
         and last syncing occurred within alloted time for all sync categories
         """
         return (
-            self.is_structure_sync_ok
-            and self.is_notification_sync_ok
-            and self.is_forwarding_sync_ok
-            and self.is_assets_sync_ok
+            self.is_structure_sync_fresh
+            and self.is_notification_sync_fresh
+            and self.is_forwarding_sync_fresh
+            and self.is_assets_sync_fresh
         )
 
+    def update_is_up(self) -> bool:
+        """Check if all services for this owner are up, notify admins if necessary
+        and store result.
+        """
+
+        def fresh_str(value: bool) -> str:
+            return "up" if value else "down"
+
+        is_up = self.are_all_syncs_ok
+        if (
+            STRUCTURES_ADMIN_NOTIFICATIONS_ENABLED
+            and self.is_included_in_service_status
+        ):
+            if self.is_up and is_up is False:
+                title = f"{__title__}: Services are down for {self}"
+                msg = (
+                    f"Structure services for {self} are down. "
+                    "Admin action is likely required to restore services.\n"
+                    f"- Structures: {fresh_str(self.is_structure_sync_fresh)}\n"
+                    f"- Notifications: {fresh_str(self.is_notification_sync_fresh)}\n"
+                    f"- Forwarding: {fresh_str(self.is_forwarding_sync_fresh)}\n"
+                    f"- Assets: {fresh_str(self.is_assets_sync_fresh)}\n"
+                )
+                notify_admins(message=msg, title=title, level="danger")
+            elif self.is_up is False and is_up:
+                title = f"{__title__}: Services restored for {self}"
+                msg = f"Structure services for {self} have been restored. "
+                notify_admins(message=msg, title=title, level="success")
+            elif self.is_up is None and is_up:
+                title = f"{__title__}: Services enabled {self}"
+                msg = f"Structure services for {self} have been enabled. "
+                notify_admins(message=msg, title=title, level="success")
+        if self.is_up != is_up:
+            self.is_up = is_up
+            self.save(update_fields=["is_up"])
+        return is_up
+
     def add_character(
         self, character_ownership: CharacterOwnership
     ) -> "OwnerCharacter":
@@ -257,71 +307,49 @@ class Owner(models.Model):
         return self.characters.count()
 
     def fetch_token(
-        self, rotate_characters: bool = False, ignore_schedule: bool = False
+        self,
+        rotate_characters: RotateCharactersType = None,
+        ignore_schedule: bool = False,
     ) -> Token:
         """Fetch a valid token for the owner and return it.
 
         Args:
-            rotate_characters: rotate through characters with every new call
+            rotate_characters: For which sync type to rotate through characters \
+                with every new call
             ignore_schedule: Ignore current schedule when rotating
 
         Raises TokenError when no valid token can be provided.
         """
-
-        def notify_error(
-            error: str, character: CharacterOwnership = None, level="warning"
-        ) -> None:
-            """Notify admin and users about an error with the owner characters."""
-            message_id = f"{__title__}-Owner-fetch_token-{self.pk}"
-            title = f"{__title__}: Failed to fetch token for {self}"
-            error = f"{error} Please add a new character to restore service level."
-            if character and character.character_ownership:
-                notify_throttled(
-                    message_id=message_id,
-                    user=character.character_ownership.user,
-                    title=title,
-                    message=error,
-                    level=level,
-                )
-                title = f"FYI: {title}"
-            notify_admins_throttled(
-                message_id=message_id,
-                title=title,
-                message=error,
-                level=level,
-                timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
-            )
-
         token = None
-        for character in self.characters.order_by("last_used_at"):
+        order_by_last_used = (
+            rotate_characters.last_used_at_name
+            if rotate_characters
+            else "notifications_last_used_at"
+        )
+        for character in self.characters.order_by(order_by_last_used):
             if (
                 character.character_ownership.character.corporation_id
                 != self.corporation.corporation_id
             ):
-                notify_error(
-                    f"{character.character_ownership}: Character does no longer belong to the owner's corporation and has been removed. ",
+                self._delete_character(
+                    ("Character does no longer belong to the owner's corporation."),
                     character,
                 )
-                character.delete()
                 continue
             elif not character.character_ownership.user.has_perm(
                 "structures.add_structure_owner"
             ):
-                notify_error(
-                    f"{character.character_ownership}: "
-                    "Character does not have sufficient permission to sync "
-                    "and has been removed."
+                self._delete_character(
+                    "Character does not have sufficient permission to sync.",
+                    character,
                 )
-                character.delete()
                 continue
             token = character.valid_token()
             if not token:
-                notify_error(
-                    f"{character.character_ownership}: Character has no valid token "
-                    "for sync and has been removed. ",
+                self._delete_character(
+                    "Character has no valid token for sync.",
                     character,
                 )
-                character.delete()
                 continue
             break  # leave the for loop if we have found a valid token
 
@@ -330,48 +358,89 @@ class Owner(models.Model):
                 f"{self}: No valid character found for sync. "
                 "Service down for this owner."
             )
-            notify_error(error, level="danger")
             raise TokenError(error)
         if rotate_characters:
-            self._rotate_character(character, ignore_schedule)
+            self._rotate_character(
+                character=character,
+                ignore_schedule=ignore_schedule,
+                rotate_characters=rotate_characters,
+            )
         return token
 
+    def _delete_character(
+        self,
+        error: str,
+        character: CharacterOwnership,
+        level="warning",
+    ) -> None:
+        """Delete character and notify it's owner and admin about the reason
+
+        Args:
+        - error: Error text
+        - character: Character this error refers to
+        - level: context level for the notification
+        - delete_character: will delete the character object if set true
+        """
+        title = f"{__title__}: {self}: Invalid character has been removed"
+        error = (
+            f"{character.character_ownership}: {error}\n"
+            "The character has been removed. "
+            "Please add a new character to restore the previous service level."
+        )
+        notify(
+            user=character.character_ownership.user,
+            title=title,
+            message=error,
+            level=level,
+        )
+        if self.characters.count() == 1:
+            error += (
+                " This owner has no configured characters anymore "
+                "and it's services are now down."
+            )
+            level = "danger"
+        notify_admins(title=f"FYI: {title}", message=error, level=level)
+        character.delete()
+
     def _rotate_character(
-        self, character: "OwnerCharacter", ignore_schedule: bool
+        self,
+        character: "OwnerCharacter",
+        ignore_schedule: bool,
+        rotate_characters: RotateCharactersType,
     ) -> None:
         """Rotate this character such that all are spread evently
-        accross the ESI cache duration for fetching notifications.
+        accross the ESI cache duration for each ESI call.
         """
         time_since_last_used = (
-            (now() - character.last_used_at).total_seconds()
-            if character.last_used_at
+            (
+                now() - getattr(character, rotate_characters.last_used_at_name)
+            ).total_seconds()
+            if getattr(character, rotate_characters.last_used_at_name)
             else None
         )
         try:
             minimum_time_between_rotations = max(
-                self.ESI_CHARACTER_NOTIFICATION_CACHE_DURATION
-                / self.characters.count(),
+                rotate_characters.esi_cache_duration / self.characters.count(),
                 60,
             )
         except ZeroDivisionError:
-            minimum_time_between_rotations = (
-                self.ESI_CHARACTER_NOTIFICATION_CACHE_DURATION
-            )
+            minimum_time_between_rotations = rotate_characters.esi_cache_duration
         if (
             ignore_schedule
             or not time_since_last_used
             or time_since_last_used >= minimum_time_between_rotations
         ):
-            character.last_used_at = now()
+            setattr(character, rotate_characters.last_used_at_name, now())
             character.save()
 
+    def _report_esi_issue(self, action: str, ex: Exception, token: Token):
+        """Report an ESI issue to admins."""
+        message = f"{self}: Failed to {action} from ESI with token {token} due to {ex}"
+        logger.warning(message, exc_info=True)
+
     def update_structures_esi(self, user: User = None):
         """Updates all structures from ESI."""
-        self.structures_last_update_ok = None
-        self.structures_last_update_at = now()
-        self.save()
-        token = self.fetch_token()
-
+        token = self.fetch_token(rotate_characters=self.RotateCharactersType.STRUCTURES)
         is_ok = self._fetch_upwell_structures(token)
         if STRUCTURES_FEATURE_CUSTOMS_OFFICES:
             is_ok &= self._fetch_custom_offices(token)
@@ -379,8 +448,8 @@ class Owner(models.Model):
             is_ok &= self._fetch_starbases(token)
 
         if is_ok:
-            self.structures_last_update_ok = True
-            self.save()
+            self.structures_last_update_at = now()
+            self.save(update_fields=["structures_last_update_at"])
             if user:
                 self._send_report_to_user(
                     topic="structures", topic_count=self.structures.count(), user=user
@@ -419,22 +488,7 @@ class Owner(models.Model):
                 has_pages=True,
             )
         except OSError as ex:
-            message_id = (
-                f"{__title__}-fetch_upwell_structures-{self.pk}-{type(ex).__name__}"
-            )
-            title = f"{__title__}: Failed to update upwell structures for {self}"
-            message = (
-                f"{self}: Failed to update upwell structures "
-                f"from ESI for due to: {ex}"
-            )
-            logger.exception(message)
-            notify_admins_throttled(
-                message_id=message_id,
-                title=title,
-                message=message,
-                level="danger",
-                timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
-            )
+            self._report_esi_issue("fetch corporation structures", ex, token)
             return False
 
         is_ok = True
@@ -464,25 +518,8 @@ class Owner(models.Model):
                     )
                     structure["position"] = structure_info["position"]
                 except OSError as ex:
-                    message_id = (
-                        f"{__title__}-fetch_upwell_structures-details-"
-                        f"{self.pk}-{type(ex).__name__}"
-                    )
-                    title = (
-                        f"{__title__}: Failed to update details for "
-                        f"structure from {self}"
-                    )
-                    message = (
-                        f"{self}: Failed to update details for structure "
-                        f"with ID {structure['structure_id']} from ESI due to: {ex}"
-                    )
-                    logger.warning(message, exc_info=True)
-                    notify_admins_throttled(
-                        message_id=message_id,
-                        title=title,
-                        message=message,
-                        level="warning",
-                        timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
+                    self._report_esi_issue(
+                        f"fetch structure #{structure['structure_id']}", ex, token
                     )
                     structure["name"] = "(no data)"
                     is_ok = False
@@ -685,19 +722,7 @@ class Owner(models.Model):
                 self._store_raw_data("customs_offices", structures, corporation_id)
 
         except OSError as ex:
-            message_id = (
-                f"{__title__}-_fetch_customs_offices-{self.pk}-{type(ex).__name__}"
-            )
-            title = f"{__title__}: Failed to update custom offices for {self}"
-            message = f"{self}: Failed to update custom offices from ESI due to: {ex}"
-            logger.exception(message)
-            notify_admins_throttled(
-                message_id=message_id,
-                title=title,
-                message=message,
-                level="danger",
-                timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
-            )
+            self._report_esi_issue("fetch custom offices", ex, token)
             return False
 
         self._remove_structures_not_returned_from_esi(
@@ -758,7 +783,6 @@ class Owner(models.Model):
 
         Return True when successful, else False.
         """
-
         structures = list()
         corporation_id = self.corporation.corporation_id
         try:
@@ -815,18 +839,22 @@ class Owner(models.Model):
             if STRUCTURES_DEVELOPER_MODE:
                 self._store_raw_data("starbases", structures, corporation_id)
 
+        except HTTPForbidden:
+            try:
+                character = self.characters.get(
+                    character_ownership__character__character_id=token.character_id
+                )
+            except ObjectDoesNotExist:
+                pass
+            else:
+                self._delete_character(
+                    "Character is not a director and therefore can not fetch starbases.",
+                    character,
+                )
+            return False
+
         except OSError as ex:
-            message_id = f"{__title__}-_fetch_starbases-{self.pk}-{type(ex).__name__}"
-            title = f"{__title__}: Failed to fetch starbases for {self}"
-            message = f"{self}: Failed to fetch starbases from ESI due to {ex}"
-            logger.exception(message)
-            notify_admins_throttled(
-                message_id=message_id,
-                title=title,
-                message=message,
-                level="danger",
-                timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
-            )
+            self._report_esi_issue("fetch starbases", ex, token)
             return False
 
         self._remove_structures_not_returned_from_esi(
@@ -862,94 +890,78 @@ class Owner(models.Model):
 
         return names
 
-    def _calc_starbase_fuel_expires(self, corporation_id, starbase, token):
+    def _calc_starbase_fuel_expires(
+        self, corporation_id: int, starbase: dict, token: Token
+    ) -> datetime:
+        """Estimate when fuel will expire for this starbase.
 
-        fuel_expires_at = None
-        if starbase["state"] != "offline":
-            starbase_details = esi_fetch(
-                "Corporation.get_corporations_corporation_id_starbases_starbase_id",
-                args={
-                    "corporation_id": corporation_id,
-                    "starbase_id": starbase["starbase_id"],
-                    "system_id": starbase["system_id"],
-                },
-                token=token,
+        Estimate will vary due to server caching of remaining fuel blocks.
+        """
+        if starbase["state"] == "offline":
+            return None
+        operation = _esi_client().Corporation.get_corporations_corporation_id_starbases_starbase_id(
+            corporation_id=corporation_id,
+            starbase_id=starbase["starbase_id"],
+            system_id=starbase["system_id"],
+            token=token.valid_access_token(),
+        )
+        operation.request_config.also_return_response = True
+        starbase_details, response = operation.result()
+        fuel_quantity = None
+        if "fuels" in starbase_details:
+            for fuel in starbase_details["fuels"]:
+                fuel_type, _ = EveType.objects.get_or_create_esi(fuel["type_id"])
+                if fuel_type.is_fuel_block:
+                    fuel_quantity = fuel["quantity"]
+        if fuel_quantity:
+            starbase_type, _ = EveType.objects.get_or_create_esi(starbase["type_id"])
+            solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
+                starbase["system_id"]
             )
-            fuel_quantity = None
-            if "fuels" in starbase_details:
-                for fuel in starbase_details["fuels"]:
-                    fuel_type, _ = EveType.objects.get_or_create_esi(fuel["type_id"])
-                    if fuel_type.is_fuel_block:
-                        fuel_quantity = fuel["quantity"]
-            if fuel_quantity:
-                starbase_type, _ = EveType.objects.get_or_create_esi(
-                    starbase["type_id"]
-                )
-                solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
-                    starbase["system_id"]
-                )
-                sov_discount = (
-                    0.25 if solar_system.corporation_has_sov(self.corporation) else 0
-                )
-                hours = math.floor(
-                    fuel_quantity
-                    / (starbase_type.starbase_fuel_per_hour * (1 - sov_discount))
-                )
-                fuel_expires_at = now() + timedelta(hours=hours)
+            sov_discount = (
+                0.25 if solar_system.corporation_has_sov(self.corporation) else 0
+            )
+            seconds = math.floor(
+                3600
+                * fuel_quantity
+                / (starbase_type.starbase_fuel_per_hour * (1 - sov_discount))
+            )
+            last_modified = parsedate_to_datetime(
+                response.headers.get("Last-Modified", format_datetime(now()))
+            )
+            fuel_expires_at = last_modified + timedelta(seconds=seconds)
 
         return fuel_expires_at
 
     def fetch_notifications_esi(self, user: User = None) -> None:
         """Fetch notifications for this owner from ESI and proceses them."""
         notifications_count_all = 0
-        self.notifications_last_update_ok = None
-        self.notifications_last_update_at = now()
-        self.save()
-        token = self.fetch_token(rotate_characters=True)
-
-        try:
-            notifications = self._fetch_notifications_from_esi(token)
-        except OSError as ex:
-            message_id = (
-                f"{__title__}-fetch_notifications-{self.pk}-{type(ex).__name__}"
+        token = self.fetch_token(
+            rotate_characters=self.RotateCharactersType.NOTIFICATIONS
+        )
+        notifications = self._fetch_notifications_from_esi(token)
+        notifications_count_new = self._store_notifications(notifications)
+        self._process_moon_notifications()
+        if notifications_count_new > 0:
+            logger.info(
+                "%s: Received %d new notifications from ESI",
+                self,
+                notifications_count_new,
             )
-            title = f"{__title__}: Failed to update notifications for {self}"
-            message = f"{self}: Failed to update notifications from ESI due to {ex}"
-            logger.exception(message)
-            notify_admins_throttled(
-                message_id=message_id,
-                title=title,
-                message=message,
-                level="danger",
-                timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
-            )
-            self.notifications_last_update_ok = False
-            self.save()
-            raise ex
+            self._process_timers_for_notifications(token)
+            notifications_count_all += notifications_count_new
         else:
-            notifications_count_new = self._store_notifications(notifications)
-            self._process_moon_notifications()
-            if notifications_count_new > 0:
-                logger.info(
-                    "%s: Received %d new notifications from ESI",
-                    self,
-                    notifications_count_new,
-                )
-                self._process_timers_for_notifications(token)
-                notifications_count_all += notifications_count_new
+            logger.info("%s: No new notifications received from ESI", self)
 
-            else:
-                logger.info("%s: No new notifications received from ESI", self)
+        self.notifications_last_update_at = now()
+        self.save(update_fields=["notifications_last_update_at"])
 
-            self.notifications_last_update_ok = True
-            self.save()
-
-            if user:
-                self._send_report_to_user(
-                    topic="notifications",
-                    topic_count=notifications_count_all,
-                    user=user,
-                )
+        if user:
+            self._send_report_to_user(
+                topic="notifications",
+                topic_count=notifications_count_all,
+                user=user,
+            )
 
     def _fetch_notifications_from_esi(self, token: Token) -> dict:
         """fetching all notifications from ESI for current owner"""
@@ -1046,8 +1058,9 @@ class Owner(models.Model):
                 hours=STRUCTURES_HOURS_UNTIL_STALE_NOTIFICATION
             )
             notifications = (
-                Notification.objects.filter(owner=self)
-                .filter(notif_type__in=NotificationType.relevant_for_timerboard)
+                self.notifications.filter(
+                    notif_type__in=NotificationType.relevant_for_timerboard
+                )
                 .exclude(is_timer_added=True)
                 .filter(timestamp__gte=cutoff_dt_for_stale)
                 .select_related("owner", "sender")
@@ -1073,8 +1086,9 @@ class Owner(models.Model):
                 empty_refineries.count(),
             )
             notifications = (
-                Notification.objects.filter(owner=self)
-                .filter(notif_type__in=NotificationType.relevant_for_moonmining)
+                self.notifications.filter(
+                    notif_type__in=NotificationType.relevant_for_moonmining
+                )
                 .select_related("owner", "sender")
                 .order_by("timestamp")
             )
@@ -1095,71 +1109,34 @@ class Owner(models.Model):
                     refinery.save()
 
     def send_new_notifications(self, user: User = None):
-        """Forward all new notification for this owner to Discord."""
+        """Forward all new notification of this owner to configured webhooks."""
         notifications_count = 0
-        self.forwarding_last_update_ok = None
-        self.forwarding_last_update_at = now()
-        self.save()
-
         cutoff_dt_for_stale = now() - timedelta(
             hours=STRUCTURES_HOURS_UNTIL_STALE_NOTIFICATION
         )
-        all_new_notifications = list(
-            Notification.objects.filter(owner=self)
-            .filter(notif_type__in=NotificationType.values)
+        all_new_notifications = (
+            self.notifications.filter(
+                notif_type__in=(
+                    Webhook.objects.enabled_notification_types()
+                    & NotificationType.relevant_for_forwarding
+                )
+            )
             .filter(is_sent=False)
             .filter(timestamp__gte=cutoff_dt_for_stale)
-            .select_related()
+            .select_related("owner", "sender", "owner__corporation")
             .order_by("timestamp")
         )
-        new_notifications_count = 0
-        active_webhooks_count = 0
-        for webhook in self.webhooks.filter(is_active=True):
-            active_webhooks_count += 1
-            new_notifications = [
-                notif
-                for notif in all_new_notifications
-                if str(notif.notif_type) in webhook.notification_types
-            ]
-            if len(new_notifications) > 0:
-                new_notifications_count += len(new_notifications)
-                logger.info(
-                    "%s: Found %d new notifications for webhook %s",
-                    self,
-                    len(new_notifications),
-                    webhook,
-                )
-                notifications_count += self._send_notifications_to_webhook(
-                    new_notifications, webhook
-                )
-
-        if active_webhooks_count == 0:
-            logger.info("%s: No active webhooks", self)
-
-        if new_notifications_count == 0:
-            logger.info("%s: No new notifications found", self)
-
-        self.forwarding_last_update_ok = True
-        self.save()
-
+        for notif in all_new_notifications:
+            notif.send_to_configured_webhooks()
+        if not all_new_notifications:
+            logger.info("%s: No new notifications found for forwarding", self)
+        self.forwarding_last_update_at = now()
+        self.save(update_fields=["forwarding_last_update_at"])
         if user:
             self._send_report_to_user(
                 topic="notifications", topic_count=notifications_count, user=user
             )
 
-    def _send_notifications_to_webhook(self, new_notifications, webhook) -> int:
-        """sends all notifications to given webhook"""
-        sent_count = 0
-        for notification in new_notifications:
-            if (
-                not notification.filter_for_npc_attacks()
-                and not notification.filter_for_alliance_level()
-            ):
-                if notification.send_to_webhook(webhook):
-                    sent_count += 1
-
-        return sent_count
-
     def _send_report_to_user(self, topic: str, topic_count: int, user: User):
         message_details = "%(count)s %(topic)s synced." % {
             "count": topic_count,
@@ -1195,37 +1172,17 @@ class Owner(models.Model):
 
     def update_asset_esi(self, user: User = None):
         """Update all assets from ESI related to active structure for this owner."""
-        self.assets_last_update_ok = None
-        self.assets_last_update_at = now()
-        self.save()
-
         token = self.fetch_token()
-        structure_ids = {x.id for x in Structure.objects.filter(owner=self)}
-        try:
-            OwnerAsset.objects.update_or_create_for_structures_esi(
-                structure_ids, self.corporation.corporation_id, token
+        structure_ids = list(self.structures.values_list("id", flat=True))
+        OwnerAsset.objects.update_or_create_for_structures_esi(
+            structure_ids, self.corporation.corporation_id, token
+        )
+        self.assets_last_update_at = now()
+        self.save(update_fields=["assets_last_update_at"])
+        if user:
+            self._send_report_to_user(
+                topic="assets", topic_count=self.structures.count(), user=user
             )
-        except OSError as ex:
-            message_id = f"{__title__}-fetch_assets-{self.pk}-{type(ex).__name__}"
-            title = f"{__title__}: Failed to update assets for {self}"
-            message = f"{self}: Failed to update assets from ESI due to {ex}"
-            logger.warning(message, exc_info=True)
-            notify_admins_throttled(
-                message_id=message_id,
-                title=title,
-                message=message,
-                level="warning",
-                timeout=STRUCTURES_NOTIFY_THROTTLED_TIMEOUT,
-            )
-            raise ex
-        else:
-            self.assets_last_update_ok = True
-            self.save()
-
-            if user:
-                self._send_report_to_user(
-                    topic="assets", topic_count=self.structures.count(), user=user
-                )
 
     @classmethod
     def get_esi_scopes(cls) -> list:
@@ -1254,12 +1211,19 @@ class OwnerCharacter(models.Model):
         related_name="+",
         help_text="character used for syncing",
     )
-    last_used_at = models.DateTimeField(
+    structures_last_used_at = models.DateTimeField(
         null=True,
         default=None,
         editable=False,
         db_index=True,
-        help_text="when this character was last used for sync",
+        help_text="when this character was last used for syncing structures",
+    )
+    notifications_last_used_at = models.DateTimeField(
+        null=True,
+        default=None,
+        editable=False,
+        db_index=True,
+        help_text="when this character was last used for syncing notifications",
     )
     created_at = models.DateTimeField(auto_now_add=True)
 
