@@ -135,14 +135,13 @@ class TestDiscordWebhookMixin(TestCase):
         self.assertEqual(result, "[test-name](test-url)")
 
 
-@patch(MODULE_PATH + ".notify_admins_throttled")
 @patch(MODULE_PATH + ".dhooks_lite.Webhook.execute")
 class TestSendTestMessage(TestCase):
     def setUp(self) -> None:
         self.webhook = Webhook("Dummy 1", "dummy-1-url")
         self.user = AuthUtils.create_user("Bruce Wayne")
 
-    def test_normal_no_user(self, mock_execute, mock_notify_admins_throttled):
+    def test_normal_no_user(self, mock_execute):
         mock_response = dhooks_lite.WebhookResponse(
             {}, status_code=200, content={"dummy": True}
         )
@@ -153,7 +152,7 @@ class TestSendTestMessage(TestCase):
         self.assertTrue(result_2)
         self.assertTrue(mock_execute.called)
 
-    def test_normal_with_user(self, mock_execute, mock_notify_admins_throttled):
+    def test_normal_with_user(self, mock_execute):
         mock_response = dhooks_lite.WebhookResponse(
             {}, status_code=200, content={"dummy": True}
         )
@@ -163,9 +162,8 @@ class TestSendTestMessage(TestCase):
         self.assertEqual(result_1, "(no info)")
         self.assertTrue(result_2)
         self.assertTrue(mock_execute.called)
-        self.assertFalse(mock_notify_admins_throttled.called)
 
-    def test_error_1(self, mock_execute, mock_notify_admins_throttled):
+    def test_error_1(self, mock_execute):
         mock_response = dhooks_lite.WebhookResponse(
             {}, status_code=404, content={"dummy": True}
         )
@@ -175,13 +173,11 @@ class TestSendTestMessage(TestCase):
         self.assertEqual(result_1, "(no info)")
         self.assertFalse(result_2)
         self.assertTrue(mock_execute.called)
-        self.assertTrue(mock_notify_admins_throttled.called)
 
-    def test_error_2(self, mock_execute, mock_notify_admins_throttled):
-        mock_execute.side_effect = RuntimeError
+    def test_error_2(self, mock_execute):
+        mock_execute.side_effect = OSError
 
         result_1, result_2 = self.webhook.send_test_message()
-        self.assertEqual(result_1, "RuntimeError")
+        self.assertEqual(result_1, "OSError")
         self.assertFalse(result_2)
         self.assertTrue(mock_execute.called)
-        self.assertFalse(mock_notify_admins_throttled.called)
